function msfAverage(block)
% Level-2 M file S-function for applying Average filtering  
% (image edge detection).

  
  setup(block);

%endfunction

function setup(block)
  
  %% Register dialog parameter:  
  block.NumDialogPrms = 1;
  block.DialogPrmsTunable = {'Tunable'};
 
  %% Register ports
  block.NumInputPorts  = 1;
  block.NumOutputPorts = 1;
  
  %% Setup port properties
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;

  block.InputPort(1).DatatypeID   = 3;
  block.InputPort(1).Complexity   = 'Real';
  block.InputPort(1).SamplingMode = 'Sample';
  block.InputPort(1).Overwritable = false; % No in-place operation
  
  block.OutputPort(1).DatatypeID   = 3;
  block.OutputPort(1).Complexity   = 'Real';
  block.OutputPort(1).SamplingMode = 'Sample';
  
  %% Register block methods (through MATLAB function handles)
  block.RegBlockMethod('Outputs', @Output);
 
%endfunction

function  g = average_filter(f, dir)
  
  %% Average filter coefficients
  h1 = [1 1 1; 1 1 1; 1 1 1];
  h2 = [1 1 1; 1 2 1; 1 1 1];
  h3 = [1 2 1; 2 4 2; 1 2 1];
  
  if dir == 1 % low
    g = abs(filter2(h3,f).*(1/16)); 
  elseif dir == 2 % med
    g = abs(filter2(h2,f).*(1/10));
  else % high
    g = abs(filter2(h1,f).*(1/9));
  end
  
  %% Prevent overflow when converting to uint8
  g(g>255) = 255;
  
  g = uint8(round(g));
  
%endfunction
  
%%
%% Block Output method: Perform average filtering
%%
function Output(block)
  
  dir = block.DialogPrm(1).Data;
  block.OutputPort(1).Data = average_filter(block.InputPort(1).Data, dir);
%endfunction
