function msfLaplace(block)
% Level-2 M file S-function for applying Laplace filtering  
% (image edge detection).

  
  setup(block);

%endfunction

function setup(block)
  
  %% Register dialog parameter: edge detection kernel
  block.NumDialogPrms = 1;
  block.DialogPrmsTunable = {'Tunable'};
 
  %% Register ports
  block.NumInputPorts  = 1;
  block.NumOutputPorts = 1;
  
  %% Setup port properties
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;

  block.InputPort(1).DatatypeID   = 3;
  block.InputPort(1).Complexity   = 'Real';
  block.InputPort(1).SamplingMode = 'Sample';
  block.InputPort(1).Overwritable = false; % No in-place operation
  
  block.OutputPort(1).DatatypeID   = 3;
  block.OutputPort(1).Complexity   = 'Real';
  block.OutputPort(1).SamplingMode = 'Sample';
  
  %% Register block methods (through MATLAB function handles)
  block.RegBlockMethod('Outputs', @Output);
 
%endfunction

function  g = laplace_filter(f, dir)
  
  %% laplace filter coefficients
  h1 = [0 1 0; 1 -4 1; 0 1 0];
  h2 = [1 1 1; 1 -8 1; 1 1 1];

  if dir == 1 % 4-neighb.
     g = abs(filter2(h1,f).*2);
  elseif dir == 2 % 8-neighb.
     g = abs(filter2(h2,f).*4);
  end
  
  %% Prevent overflow when converting to uint8
  g(g>255) = 255;
  
  g = uint8(round(g));
  
%endfunction
  
%%
%% Block Output method: Perform laplace filtering
%%
function Output(block)
  
  dir = block.DialogPrm(1).Data;
  block.OutputPort(1).Data = laplace_filter(block.InputPort(1).Data, dir);
%endfunction
